@@ -4,7 +4,7 @@
 # may not use this file except in compliance with the License.  You
 # may obtain a copy of the License at
 #
-# http://www.apache.org/licenses/LICENSE-2.0
+# https://www.apache.org/licenses/LICENSE-2.0
 #
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
@@ -12,14 +12,28 @@
 # implied.  See the License for the specific language governing
 # permissions and limitations under the License.
 
-"""Represent a deployment of MongoDB servers."""
+"""Represent a deployment of MongoDB servers.
+
+.. seealso:: This module is compatible with both the synchronous and asynchronous PyMongo APIs.
+"""
+from __future__ import annotations
 
 from random import sample
-from typing import Any, Callable, Dict, List, NamedTuple, Optional, Tuple
+from typing import (
+    Any,
+    Callable,
+    List,
+    Mapping,
+    MutableMapping,
+    NamedTuple,
+    Optional,
+    cast,
+)
 
+from bson.min_key import MinKey
 from bson.objectid import ObjectId
 from pymongo import common
-from pymongo.errors import ConfigurationError
+from pymongo.errors import ConfigurationError, PyMongoError
 from pymongo.read_preferences import ReadPreference, _AggWritePref, _ServerMode
 from pymongo.server_description import ServerDescription
 from pymongo.server_selectors import Selection
@@ -40,17 +54,17 @@ class _TopologyType(NamedTuple):
 TOPOLOGY_TYPE = _TopologyType(*range(6))
 
 # Topologies compatible with SRV record polling.
-SRV_POLLING_TOPOLOGIES: Tuple[int, int] = (TOPOLOGY_TYPE.Unknown, TOPOLOGY_TYPE.Sharded)
+SRV_POLLING_TOPOLOGIES: tuple[int, int] = (TOPOLOGY_TYPE.Unknown, TOPOLOGY_TYPE.Sharded)
 
 
 _ServerSelector = Callable[[List[ServerDescription]], List[ServerDescription]]
 
 
-class TopologyDescription(object):
+class TopologyDescription:
     def __init__(
         self,
         topology_type: int,
-        server_descriptions: Dict[_Address, ServerDescription],
+        server_descriptions: dict[_Address, ServerDescription],
         replica_set_name: Optional[str],
         max_set_version: Optional[int],
         max_election_id: Optional[ObjectId],
@@ -58,14 +72,13 @@ class TopologyDescription(object):
     ) -> None:
         """Representation of a deployment of MongoDB servers.
 
-        :Parameters:
-          - `topology_type`: initial type
-          - `server_descriptions`: dict of (address, ServerDescription) for
+        :param topology_type: initial type
+        :param server_descriptions: dict of (address, ServerDescription) for
             all seeds
-          - `replica_set_name`: replica set name or None
-          - `max_set_version`: greatest setVersion seen from a primary, or None
-          - `max_election_id`: greatest electionId seen from a primary, or None
-          - `topology_settings`: a TopologySettings
+        :param replica_set_name: replica set name or None
+        :param max_set_version: greatest setVersion seen from a primary, or None
+        :param max_election_id: greatest electionId seen from a primary, or None
+        :param topology_settings: a TopologySettings
         """
         self._topology_type = topology_type
         self._replica_set_name = replica_set_name
@@ -98,7 +111,7 @@ class TopologyDescription(object):
                 s.logical_session_timeout_minutes for s in readable_servers
             )
 
-    def _init_incompatible_err(self):
+    def _init_incompatible_err(self) -> None:
         """Internal compatibility check for non-load balanced topologies."""
         for s in self._server_descriptions.values():
             if not s.is_server_type_known:
@@ -157,12 +170,12 @@ class TopologyDescription(object):
     def has_server(self, address: _Address) -> bool:
         return address in self._server_descriptions
 
-    def reset_server(self, address: _Address) -> "TopologyDescription":
+    def reset_server(self, address: _Address) -> TopologyDescription:
         """A copy of this description, with one server marked Unknown."""
         unknown_sd = self._server_descriptions[address].to_unknown()
         return updated_topology_description(self, unknown_sd)
 
-    def reset(self) -> "TopologyDescription":
+    def reset(self) -> TopologyDescription:
         """A copy of this description, with all servers marked Unknown."""
         if self._topology_type == TOPOLOGY_TYPE.ReplicaSetWithPrimary:
             topology_type = TOPOLOGY_TYPE.ReplicaSetNoPrimary
@@ -170,7 +183,7 @@ class TopologyDescription(object):
             topology_type = self._topology_type
 
         # The default ServerDescription's type is Unknown.
-        sds = dict((address, ServerDescription(address)) for address in self._server_descriptions)
+        sds = {address: ServerDescription(address) for address in self._server_descriptions}
 
         return TopologyDescription(
             topology_type,
@@ -181,9 +194,10 @@ class TopologyDescription(object):
             self._topology_settings,
         )
 
-    def server_descriptions(self) -> Dict[_Address, ServerDescription]:
-        """Dict of (address,
-        :class:`~pymongo.server_description.ServerDescription`)."""
+    def server_descriptions(self) -> dict[_Address, ServerDescription]:
+        """dict of (address,
+        :class:`~pymongo.server_description.ServerDescription`).
+        """
         return self._server_descriptions.copy()
 
     @property
@@ -220,7 +234,7 @@ class TopologyDescription(object):
         return self._ls_timeout_minutes
 
     @property
-    def known_servers(self) -> List[ServerDescription]:
+    def known_servers(self) -> list[ServerDescription]:
         """List of Servers of types besides Unknown."""
         return [s for s in self._server_descriptions.values() if s.is_server_type_known]
 
@@ -230,7 +244,7 @@ class TopologyDescription(object):
         return any(s for s in self._server_descriptions.values() if s.is_server_type_known)
 
     @property
-    def readable_servers(self) -> List[ServerDescription]:
+    def readable_servers(self) -> list[ServerDescription]:
         """List of readable Servers."""
         return [s for s in self._server_descriptions.values() if s.is_readable]
 
@@ -251,14 +265,22 @@ class TopologyDescription(object):
     def srv_max_hosts(self) -> int:
         return self._topology_settings._srv_max_hosts
 
-    def _apply_local_threshold(self, selection):
+    def _apply_local_threshold(self, selection: Optional[Selection]) -> list[ServerDescription]:
         if not selection:
             return []
+        round_trip_times: list[float] = []
+        for server in selection.server_descriptions:
+            if server.round_trip_time is None:
+                config_err_msg = f"round_trip_time for server {server.address} is unexpectedly None: {self}, servers: {selection.server_descriptions}"
+                raise ConfigurationError(config_err_msg)
+            round_trip_times.append(server.round_trip_time)
         # Round trip time in seconds.
-        fastest = min(s.round_trip_time for s in selection.server_descriptions)
+        fastest = min(round_trip_times)
         threshold = self._topology_settings.local_threshold_ms / 1000.0
         return [
-            s for s in selection.server_descriptions if (s.round_trip_time - fastest) <= threshold
+            s
+            for s in selection.server_descriptions
+            if (cast(float, s.round_trip_time) - fastest) <= threshold
         ]
 
     def apply_selector(
@@ -266,15 +288,14 @@ class TopologyDescription(object):
         selector: Any,
         address: Optional[_Address] = None,
         custom_selector: Optional[_ServerSelector] = None,
-    ) -> List[ServerDescription]:
+    ) -> list[ServerDescription]:
         """List of servers matching the provided selector(s).
 
-        :Parameters:
-          - `selector`: a callable that takes a Selection as input and returns
+        :param selector: a callable that takes a Selection as input and returns
             a Selection as output. For example, an instance of a read
             preference from :mod:`~pymongo.read_preferences`.
-          - `address` (optional): A server address to select.
-          - `custom_selector` (optional): A callable that augments server
+        :param address: A server address to select.
+        :param custom_selector: A callable that augments server
             selection rules. Accepts a list of
             :class:`~pymongo.server_description.ServerDescription` objects and
             return a list of server descriptions that should be considered
@@ -319,8 +340,7 @@ class TopologyDescription(object):
         """Does this topology have any readable servers available matching the
         given read preference?
 
-        :Parameters:
-          - `read_preference`: an instance of a read preference from
+        :param read_preference: an instance of a read preference from
             :mod:`~pymongo.read_preferences`. Defaults to
             :attr:`~pymongo.read_preferences.ReadPreference.PRIMARY`.
 
@@ -342,10 +362,10 @@ class TopologyDescription(object):
         """
         return self.has_readable_server(ReadPreference.PRIMARY)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         # Sort the servers by address.
         servers = sorted(self._server_descriptions.values(), key=lambda sd: sd.address)
-        return "<%s id: %s, topology_type: %s, servers: %r>" % (
+        return "<{} id: {}, topology_type: {}, servers: {!r}>".format(
             self.__class__.__name__,
             self._topology_settings._topology_id,
             self.topology_type_name,
@@ -367,12 +387,11 @@ _SERVER_TYPE_TO_TOPOLOGY_TYPE = {
 
 def updated_topology_description(
     topology_description: TopologyDescription, server_description: ServerDescription
-) -> "TopologyDescription":
+) -> TopologyDescription:
     """Return an updated copy of a TopologyDescription.
 
-    :Parameters:
-      - `topology_description`: the current TopologyDescription
-      - `server_description`: a new ServerDescription that resulted from
+    :param topology_description: the current TopologyDescription
+    :param server_description: a new ServerDescription that resulted from
         a hello call
 
     Called after attempting (successfully or not) to call hello on the
@@ -399,8 +418,9 @@ def updated_topology_description(
         if set_name is not None and set_name != server_description.replica_set_name:
             error = ConfigurationError(
                 "client is configured to connect to a replica set named "
-                "'%s' but this node belongs to a set named '%s'"
-                % (set_name, server_description.replica_set_name)
+                "'{}' but this node belongs to a set named '{}'".format(
+                    set_name, server_description.replica_set_name
+                )
             )
             sds[address] = server_description.to_unknown(error=error)
         # Single type never changes.
@@ -469,14 +489,16 @@ def updated_topology_description(
     )
 
 
-def _updated_topology_description_srv_polling(topology_description, seedlist):
+def _updated_topology_description_srv_polling(
+    topology_description: TopologyDescription, seedlist: list[tuple[str, Any]]
+) -> TopologyDescription:
     """Return an updated copy of a TopologyDescription.
 
-    :Parameters:
-      - `topology_description`: the current TopologyDescription
-      - `seedlist`: a list of new seeds new ServerDescription that resulted from
+    :param topology_description: the current TopologyDescription
+    :param seedlist: a list of new seeds new ServerDescription that resulted from
         a hello call
     """
+    assert topology_description.topology_type in SRV_POLLING_TOPOLOGIES
     # Create a copy of the server descriptions.
     sds = topology_description.server_descriptions()
 
@@ -493,7 +515,7 @@ def _updated_topology_description_srv_polling(topology_description, seedlist):
         new_hosts = set(seedlist) - set(sds.keys())
         n_to_add = topology_description.srv_max_hosts - len(sds)
         if n_to_add > 0:
-            seedlist = sample(new_hosts, min(n_to_add, len(new_hosts)))
+            seedlist = sample(sorted(new_hosts), min(n_to_add, len(new_hosts)))
         else:
             seedlist = []
     # Add SDs corresponding to servers recently added to the SRV record.
@@ -511,8 +533,12 @@ def _updated_topology_description_srv_polling(topology_description, seedlist):
 
 
 def _update_rs_from_primary(
-    sds, replica_set_name, server_description, max_set_version, max_election_id
-):
+    sds: MutableMapping[_Address, ServerDescription],
+    replica_set_name: Optional[str],
+    server_description: ServerDescription,
+    max_set_version: Optional[int],
+    max_election_id: Optional[ObjectId],
+) -> tuple[int, Optional[str], Optional[int], Optional[ObjectId]]:
     """Update topology description from a primary's hello response.
 
     Pass in a dict of ServerDescriptions, current replica set name, the
@@ -529,26 +555,42 @@ def _update_rs_from_primary(
         # We found a primary but it doesn't have the replica_set_name
         # provided by the user.
         sds.pop(server_description.address)
-        return (_check_has_primary(sds), replica_set_name, max_set_version, max_election_id)
+        return _check_has_primary(sds), replica_set_name, max_set_version, max_election_id
 
-    max_election_tuple = max_set_version, max_election_id
-    if None not in server_description.election_tuple:
-        if (
-            None not in max_election_tuple
-            and max_election_tuple > server_description.election_tuple
+    if server_description.max_wire_version is None or server_description.max_wire_version < 17:
+        new_election_tuple: tuple = (server_description.set_version, server_description.election_id)
+        max_election_tuple: tuple = (max_set_version, max_election_id)
+        if None not in new_election_tuple:
+            if None not in max_election_tuple and new_election_tuple < max_election_tuple:
+                # Stale primary, set to type Unknown.
+                sds[server_description.address] = server_description.to_unknown(
+                    PyMongoError(
+                        f"primary marked stale due to electionId/setVersion mismatch, {new_election_tuple} is stale compared to {max_election_tuple}"
+                    )
+                )
+                return _check_has_primary(sds), replica_set_name, max_set_version, max_election_id
+            max_election_id = server_description.election_id
+
+        if server_description.set_version is not None and (
+            max_set_version is None or server_description.set_version > max_set_version
         ):
-
+            max_set_version = server_description.set_version
+    else:
+        new_election_tuple = server_description.election_id, server_description.set_version
+        max_election_tuple = max_election_id, max_set_version
+        new_election_safe = tuple(MinKey() if i is None else i for i in new_election_tuple)
+        max_election_safe = tuple(MinKey() if i is None else i for i in max_election_tuple)
+        if new_election_safe < max_election_safe:
             # Stale primary, set to type Unknown.
-            sds[server_description.address] = server_description.to_unknown()
-            return (_check_has_primary(sds), replica_set_name, max_set_version, max_election_id)
-
-        max_election_id = server_description.election_id
-
-    if server_description.set_version is not None and (
-        max_set_version is None or server_description.set_version > max_set_version
-    ):
-
-        max_set_version = server_description.set_version
+            sds[server_description.address] = server_description.to_unknown(
+                PyMongoError(
+                    f"primary marked stale due to electionId/setVersion mismatch, {new_election_tuple} is stale compared to {max_election_tuple}"
+                )
+            )
+            return _check_has_primary(sds), replica_set_name, max_set_version, max_election_id
+        else:
+            max_election_id = server_description.election_id
+            max_set_version = server_description.set_version
 
     # We've heard from the primary. Is it the same primary as before?
     for server in sds.values():
@@ -556,9 +598,10 @@ def _update_rs_from_primary(
             server.server_type is SERVER_TYPE.RSPrimary
             and server.address != server_description.address
         ):
-
             # Reset old primary's type to Unknown.
-            sds[server.address] = server.to_unknown()
+            sds[server.address] = server.to_unknown(
+                PyMongoError("primary marked stale due to discovery of newer primary")
+            )
 
             # There can be only one prior primary.
             break
@@ -577,7 +620,11 @@ def _update_rs_from_primary(
     return (_check_has_primary(sds), replica_set_name, max_set_version, max_election_id)
 
 
-def _update_rs_with_primary_from_member(sds, replica_set_name, server_description):
+def _update_rs_with_primary_from_member(
+    sds: MutableMapping[_Address, ServerDescription],
+    replica_set_name: Optional[str],
+    server_description: ServerDescription,
+) -> int:
     """RS with known primary. Process a response from a non-primary.
 
     Pass in a dict of ServerDescriptions, current replica set name, and the
@@ -596,7 +643,11 @@ def _update_rs_with_primary_from_member(sds, replica_set_name, server_descriptio
     return _check_has_primary(sds)
 
 
-def _update_rs_no_primary_from_member(sds, replica_set_name, server_description):
+def _update_rs_no_primary_from_member(
+    sds: MutableMapping[_Address, ServerDescription],
+    replica_set_name: Optional[str],
+    server_description: ServerDescription,
+) -> tuple[int, Optional[str]]:
     """RS without known primary. Update from a non-primary's response.
 
     Pass in a dict of ServerDescriptions, current replica set name, and the
@@ -624,7 +675,7 @@ def _update_rs_no_primary_from_member(sds, replica_set_name, server_description)
     return topology_type, replica_set_name
 
 
-def _check_has_primary(sds):
+def _check_has_primary(sds: Mapping[_Address, ServerDescription]) -> int:
     """Current topology type is ReplicaSetWithPrimary. Is primary still known?
 
     Pass in a dict of ServerDescriptions.
@@ -634,5 +685,5 @@ def _check_has_primary(sds):
     for s in sds.values():
         if s.server_type == SERVER_TYPE.RSPrimary:
             return TOPOLOGY_TYPE.ReplicaSetWithPrimary
-    else:
+    else:  # noqa: PLW0120
         return TOPOLOGY_TYPE.ReplicaSetNoPrimary
