@@ -16,18 +16,27 @@
 from __future__ import annotations
 
 import os
+import pathlib
 import socket
 import sys
 
 sys.path[0:0] = [""]
 
-from test import HAVE_IPADDRESS, IntegrationTest, SkipTest, client_context, unittest
-from test.utils import (
-    EventListener,
-    cat_files,
+from test import (
+    HAVE_IPADDRESS,
+    IntegrationTest,
+    PyMongoTestCase,
+    SkipTest,
+    client_context,
     connected,
-    ignore_deprecations,
     remove_all_users,
+    unittest,
+)
+from test.utils_shared import (
+    EventListener,
+    OvertCommandListener,
+    cat_files,
+    ignore_deprecations,
 )
 from urllib.parse import quote_plus
 
@@ -57,7 +66,13 @@ except ImportError:
 if HAVE_SSL:
     import ssl
 
-CERT_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), "certificates")
+_IS_SYNC = True
+
+if _IS_SYNC:
+    CERT_PATH = os.path.join(pathlib.Path(__file__).resolve().parent, "certificates")
+else:
+    CERT_PATH = os.path.join(pathlib.Path(__file__).resolve().parent.parent, "certificates")
+
 CLIENT_PEM = os.path.join(CERT_PATH, "client.pem")
 CLIENT_ENCRYPTED_PEM = os.path.join(CERT_PATH, "password_protected.pem")
 CA_PEM = os.path.join(CERT_PATH, "ca.pem")
@@ -76,45 +91,45 @@ MONGODB_X509_USERNAME = "C=US,ST=New York,L=New York City,O=MDB,OU=Drivers,CN=cl
 # use 'localhost' for the hostname of all hosts.
 
 
-class TestClientSSL(unittest.TestCase):
+class TestClientSSL(PyMongoTestCase):
     @unittest.skipIf(HAVE_SSL, "The ssl module is available, can't test what happens without it.")
     def test_no_ssl_module(self):
         # Explicit
-        self.assertRaises(ConfigurationError, MongoClient, ssl=True)
+        self.assertRaises(ConfigurationError, self.simple_client, ssl=True)
 
         # Implied
-        self.assertRaises(ConfigurationError, MongoClient, tlsCertificateKeyFile=CLIENT_PEM)
+        self.assertRaises(ConfigurationError, self.simple_client, tlsCertificateKeyFile=CLIENT_PEM)
 
     @unittest.skipUnless(HAVE_SSL, "The ssl module is not available.")
     @ignore_deprecations
     def test_config_ssl(self):
         # Tests various ssl configurations
-        self.assertRaises(ValueError, MongoClient, ssl="foo")
+        self.assertRaises(ValueError, self.simple_client, ssl="foo")
         self.assertRaises(
-            ConfigurationError, MongoClient, tls=False, tlsCertificateKeyFile=CLIENT_PEM
+            ConfigurationError, self.simple_client, tls=False, tlsCertificateKeyFile=CLIENT_PEM
         )
-        self.assertRaises(TypeError, MongoClient, ssl=0)
-        self.assertRaises(TypeError, MongoClient, ssl=5.5)
-        self.assertRaises(TypeError, MongoClient, ssl=[])
+        self.assertRaises(TypeError, self.simple_client, ssl=0)
+        self.assertRaises(TypeError, self.simple_client, ssl=5.5)
+        self.assertRaises(TypeError, self.simple_client, ssl=[])
 
-        self.assertRaises(IOError, MongoClient, tlsCertificateKeyFile="NoSuchFile")
-        self.assertRaises(TypeError, MongoClient, tlsCertificateKeyFile=True)
-        self.assertRaises(TypeError, MongoClient, tlsCertificateKeyFile=[])
+        self.assertRaises(IOError, self.simple_client, tlsCertificateKeyFile="NoSuchFile")
+        self.assertRaises(TypeError, self.simple_client, tlsCertificateKeyFile=True)
+        self.assertRaises(TypeError, self.simple_client, tlsCertificateKeyFile=[])
 
         # Test invalid combinations
         self.assertRaises(
-            ConfigurationError, MongoClient, tls=False, tlsCertificateKeyFile=CLIENT_PEM
+            ConfigurationError, self.simple_client, tls=False, tlsCertificateKeyFile=CLIENT_PEM
         )
-        self.assertRaises(ConfigurationError, MongoClient, tls=False, tlsCAFile=CA_PEM)
-        self.assertRaises(ConfigurationError, MongoClient, tls=False, tlsCRLFile=CRL_PEM)
+        self.assertRaises(ConfigurationError, self.simple_client, tls=False, tlsCAFile=CA_PEM)
+        self.assertRaises(ConfigurationError, self.simple_client, tls=False, tlsCRLFile=CRL_PEM)
         self.assertRaises(
-            ConfigurationError, MongoClient, tls=False, tlsAllowInvalidCertificates=False
+            ConfigurationError, self.simple_client, tls=False, tlsAllowInvalidCertificates=False
         )
         self.assertRaises(
-            ConfigurationError, MongoClient, tls=False, tlsAllowInvalidHostnames=False
+            ConfigurationError, self.simple_client, tls=False, tlsAllowInvalidHostnames=False
         )
         self.assertRaises(
-            ConfigurationError, MongoClient, tls=False, tlsDisableOCSPEndpointCheck=False
+            ConfigurationError, self.simple_client, tls=False, tlsDisableOCSPEndpointCheck=False
         )
 
     @unittest.skipUnless(_HAVE_PYOPENSSL, "PyOpenSSL is not available.")
@@ -136,21 +151,18 @@ class TestSSL(IntegrationTest):
         )
         coll.drop()
         coll.insert_one({"ssl": True})
-        self.assertTrue(coll.find_one()["ssl"])
+        self.assertTrue((coll.find_one())["ssl"])
         coll.drop()
 
-    @classmethod
     @unittest.skipUnless(HAVE_SSL, "The ssl module is not available.")
-    def setUpClass(cls):
-        super().setUpClass()
+    def setUp(self):
+        super().setUp()
         # MongoClient should connect to the primary by default.
-        cls.saved_port = MongoClient.PORT
+        self.saved_port = MongoClient.PORT
         MongoClient.PORT = client_context.port
 
-    @classmethod
-    def tearDownClass(cls):
-        MongoClient.PORT = cls.saved_port
-        super().tearDownClass()
+    def tearDown(self):
+        MongoClient.PORT = self.saved_port
 
     @client_context.require_tls
     def test_simple_ssl(self):
@@ -168,7 +180,7 @@ class TestSSL(IntegrationTest):
         if not hasattr(ssl, "SSLContext") and not _ssl.IS_PYOPENSSL:
             self.assertRaises(
                 ConfigurationError,
-                MongoClient,
+                self.simple_client,
                 "localhost",
                 ssl=True,
                 tlsCertificateKeyFile=CLIENT_ENCRYPTED_PEM,
@@ -178,7 +190,7 @@ class TestSSL(IntegrationTest):
             )
         else:
             connected(
-                MongoClient(
+                self.simple_client(
                     "localhost",
                     ssl=True,
                     tlsCertificateKeyFile=CLIENT_ENCRYPTED_PEM,
@@ -195,7 +207,7 @@ class TestSSL(IntegrationTest):
                 "&tlsCAFile=%s&serverSelectionTimeoutMS=5000"
             )
             connected(
-                MongoClient(uri_fmt % (CLIENT_ENCRYPTED_PEM, CA_PEM), **self.credentials)  # type: ignore[arg-type]
+                self.simple_client(uri_fmt % (CLIENT_ENCRYPTED_PEM, CA_PEM), **self.credentials)  # type: ignore[arg-type]
             )
 
     @client_context.require_tlsCertificateKeyFile
@@ -209,7 +221,7 @@ class TestSSL(IntegrationTest):
         #
 
         # test that setting tlsCertificateKeyFile causes ssl to be set to True
-        client = MongoClient(
+        client = self.simple_client(
             client_context.host,
             client_context.port,
             tlsAllowInvalidCertificates=True,
@@ -217,7 +229,7 @@ class TestSSL(IntegrationTest):
         )
         response = client.admin.command(HelloCompat.LEGACY_CMD)
         if "setName" in response:
-            client = MongoClient(
+            client = self.simple_client(
                 client_context.pair,
                 replicaSet=response["setName"],
                 w=len(response["hosts"]),
@@ -236,7 +248,7 @@ class TestSSL(IntegrationTest):
         #   --sslPEMKeyFile=/path/to/pymongo/test/certificates/server.pem
         #   --sslCAFile=/path/to/pymongo/test/certificates/ca.pem
         #
-        client = MongoClient(
+        client = self.simple_client(
             "localhost",
             ssl=True,
             tlsCertificateKeyFile=CLIENT_PEM,
@@ -251,7 +263,7 @@ class TestSSL(IntegrationTest):
                     "Cannot validate hostname in the certificate"
                 )
 
-            client = MongoClient(
+            client = self.simple_client(
                 "localhost",
                 replicaSet=response["setName"],
                 w=len(response["hosts"]),
@@ -264,7 +276,7 @@ class TestSSL(IntegrationTest):
         self.assertClientWorks(client)
 
         if HAVE_IPADDRESS:
-            client = MongoClient(
+            client = self.simple_client(
                 "127.0.0.1",
                 ssl=True,
                 tlsCertificateKeyFile=CLIENT_PEM,
@@ -286,7 +298,7 @@ class TestSSL(IntegrationTest):
             "mongodb://localhost/?ssl=true&tlsCertificateKeyFile=%s&tlsAllowInvalidCertificates"
             "=%s&tlsCAFile=%s&tlsAllowInvalidHostnames=false"
         )
-        client = MongoClient(uri_fmt % (CLIENT_PEM, "true", CA_PEM))
+        client = self.simple_client(uri_fmt % (CLIENT_PEM, "true", CA_PEM))
         self.assertClientWorks(client)
 
     @client_context.require_tlsCertificateKeyFile
@@ -310,7 +322,7 @@ class TestSSL(IntegrationTest):
 
         with self.assertRaises(ConnectionFailure):
             connected(
-                MongoClient(
+                self.simple_client(
                     "server",
                     ssl=True,
                     tlsCertificateKeyFile=CLIENT_PEM,
@@ -322,7 +334,7 @@ class TestSSL(IntegrationTest):
             )
 
         connected(
-            MongoClient(
+            self.simple_client(
                 "server",
                 ssl=True,
                 tlsCertificateKeyFile=CLIENT_PEM,
@@ -337,7 +349,7 @@ class TestSSL(IntegrationTest):
         if "setName" in response:
             with self.assertRaises(ConnectionFailure):
                 connected(
-                    MongoClient(
+                    self.simple_client(
                         "server",
                         replicaSet=response["setName"],
                         ssl=True,
@@ -350,7 +362,7 @@ class TestSSL(IntegrationTest):
                 )
 
             connected(
-                MongoClient(
+                self.simple_client(
                     "server",
                     replicaSet=response["setName"],
                     ssl=True,
@@ -369,7 +381,7 @@ class TestSSL(IntegrationTest):
         if not hasattr(ssl, "VERIFY_CRL_CHECK_LEAF") or _ssl.IS_PYOPENSSL:
             self.assertRaises(
                 ConfigurationError,
-                MongoClient,
+                self.simple_client,
                 "localhost",
                 ssl=True,
                 tlsCAFile=CA_PEM,
@@ -378,7 +390,7 @@ class TestSSL(IntegrationTest):
             )
         else:
             connected(
-                MongoClient(
+                self.simple_client(
                     "localhost",
                     ssl=True,
                     tlsCAFile=CA_PEM,
@@ -389,7 +401,7 @@ class TestSSL(IntegrationTest):
 
             with self.assertRaises(ConnectionFailure):
                 connected(
-                    MongoClient(
+                    self.simple_client(
                         "localhost",
                         ssl=True,
                         tlsCAFile=CA_PEM,
@@ -400,7 +412,7 @@ class TestSSL(IntegrationTest):
                 )
 
             uri_fmt = "mongodb://localhost/?ssl=true&tlsCAFile=%s&serverSelectionTimeoutMS=1000"
-            connected(MongoClient(uri_fmt % (CA_PEM,), **self.credentials))  # type: ignore
+            connected(self.simple_client(uri_fmt % (CA_PEM,), **self.credentials))  # type: ignore
 
             uri_fmt = (
                 "mongodb://localhost/?ssl=true&tlsCRLFile=%s"
@@ -408,7 +420,7 @@ class TestSSL(IntegrationTest):
             )
             with self.assertRaises(ConnectionFailure):
                 connected(
-                    MongoClient(uri_fmt % (CRL_PEM, CA_PEM), **self.credentials)  # type: ignore[arg-type]
+                    self.simple_client(uri_fmt % (CRL_PEM, CA_PEM), **self.credentials)  # type: ignore[arg-type]
                 )
 
     @client_context.require_tlsCertificateKeyFile
@@ -425,12 +437,14 @@ class TestSSL(IntegrationTest):
         with self.assertRaises(ConnectionFailure):
             # Server cert is verified but hostname matching fails
             connected(
-                MongoClient("server", ssl=True, serverSelectionTimeoutMS=1000, **self.credentials)  # type: ignore[arg-type]
+                self.simple_client(
+                    "server", ssl=True, serverSelectionTimeoutMS=1000, **self.credentials
+                )  # type: ignore[arg-type]
             )
 
         # Server cert is verified. Disable hostname matching.
         connected(
-            MongoClient(
+            self.simple_client(
                 "server",
                 ssl=True,
                 tlsAllowInvalidHostnames=True,
@@ -441,12 +455,14 @@ class TestSSL(IntegrationTest):
 
         # Server cert and hostname are verified.
         connected(
-            MongoClient("localhost", ssl=True, serverSelectionTimeoutMS=1000, **self.credentials)  # type: ignore[arg-type]
+            self.simple_client(
+                "localhost", ssl=True, serverSelectionTimeoutMS=1000, **self.credentials
+            )  # type: ignore[arg-type]
         )
 
         # Server cert and hostname are verified.
         connected(
-            MongoClient(
+            self.simple_client(
                 "mongodb://localhost/?ssl=true&serverSelectionTimeoutMS=1000",
                 **self.credentials,  # type: ignore[arg-type]
             )
@@ -466,7 +482,7 @@ class TestSSL(IntegrationTest):
         ssl_support.HAVE_WINCERTSTORE = False
         try:
             with self.assertRaises(ConfigurationError):
-                MongoClient("mongodb://localhost/?ssl=true")
+                self.simple_client("mongodb://localhost/?ssl=true")
         finally:
             ssl_support.HAVE_CERTIFI = have_certifi
             ssl_support.HAVE_WINCERTSTORE = have_wincertstore
@@ -530,19 +546,18 @@ class TestSSL(IntegrationTest):
             ],
         )
 
-        noauth = MongoClient(
+        noauth = self.simple_client(
             client_context.pair,
             ssl=True,
             tlsAllowInvalidCertificates=True,
             tlsCertificateKeyFile=CLIENT_PEM,
         )
-        self.addCleanup(noauth.close)
 
         with self.assertRaises(OperationFailure):
             noauth.pymongo_test.test.find_one()
 
         listener = EventListener()
-        auth = MongoClient(
+        auth = self.simple_client(
             client_context.pair,
             authMechanism="MONGODB-X509",
             ssl=True,
@@ -550,7 +565,6 @@ class TestSSL(IntegrationTest):
             tlsCertificateKeyFile=CLIENT_PEM,
             event_listeners=[listener],
         )
-        self.addCleanup(auth.close)
 
         # No error
         auth.pymongo_test.test.find_one()
@@ -566,18 +580,16 @@ class TestSSL(IntegrationTest):
             host,
             port,
         )
-        client = MongoClient(
+        client = self.simple_client(
             uri, ssl=True, tlsAllowInvalidCertificates=True, tlsCertificateKeyFile=CLIENT_PEM
         )
-        self.addCleanup(client.close)
         # No error
         client.pymongo_test.test.find_one()
 
         uri = "mongodb://%s:%d/?authMechanism=MONGODB-X509" % (host, port)
-        client = MongoClient(
+        client = self.simple_client(
             uri, ssl=True, tlsAllowInvalidCertificates=True, tlsCertificateKeyFile=CLIENT_PEM
         )
-        self.addCleanup(client.close)
         # No error
         client.pymongo_test.test.find_one()
         # Auth should fail if username and certificate do not match
@@ -587,15 +599,14 @@ class TestSSL(IntegrationTest):
             port,
         )
 
-        bad_client = MongoClient(
+        bad_client = self.simple_client(
             uri, ssl=True, tlsAllowInvalidCertificates=True, tlsCertificateKeyFile=CLIENT_PEM
         )
-        self.addCleanup(bad_client.close)
 
         with self.assertRaises(OperationFailure):
             bad_client.pymongo_test.test.find_one()
 
-        bad_client = MongoClient(
+        bad_client = self.simple_client(
             client_context.pair,
             username="not the username",
             authMechanism="MONGODB-X509",
@@ -603,7 +614,6 @@ class TestSSL(IntegrationTest):
             tlsAllowInvalidCertificates=True,
             tlsCertificateKeyFile=CLIENT_PEM,
         )
-        self.addCleanup(bad_client.close)
 
         with self.assertRaises(OperationFailure):
             bad_client.pymongo_test.test.find_one()
@@ -616,7 +626,7 @@ class TestSSL(IntegrationTest):
         )
         try:
             connected(
-                MongoClient(
+                self.simple_client(
                     uri,
                     ssl=True,
                     tlsAllowInvalidCertificates=True,
@@ -642,7 +652,7 @@ class TestSSL(IntegrationTest):
         self.addCleanup(remove, temp_ca_bundle)
         # Add the CA cert file to the bundle.
         cat_files(temp_ca_bundle, CA_BUNDLE_PEM, CA_PEM)
-        with MongoClient(
+        with self.simple_client(
             "localhost", tls=True, tlsCertificateKeyFile=CLIENT_PEM, tlsCAFile=temp_ca_bundle
         ) as client:
             self.assertTrue(client.admin.command("ping"))
