@@ -15,30 +15,24 @@
 """Execute Transactions Spec tests."""
 from __future__ import annotations
 
-import os
 import sys
 from io import BytesIO
+from test.utils_spec_runner import SpecRunner
+
+from gridfs.synchronous.grid_file import GridFS, GridFSBucket
 
 sys.path[0:0] = [""]
 
-from test import client_context, unittest
-from test.utils import (
+from test import IntegrationTest, client_context, unittest
+from test.utils_shared import (
     OvertCommandListener,
-    SpecTestCreator,
-    rs_client,
-    single_client,
     wait_until,
 )
-from test.utils_spec_runner import SpecRunner
 from typing import List
 
 from bson import encode
 from bson.raw_bson import RawBSONDocument
-from gridfs import GridFS, GridFSBucket
-from pymongo import WriteConcern, client_session
-from pymongo.client_session import TransactionOptions
-from pymongo.command_cursor import CommandCursor
-from pymongo.cursor import Cursor
+from pymongo import WriteConcern, _csot
 from pymongo.errors import (
     CollectionInvalid,
     ConfigurationError,
@@ -49,8 +43,13 @@ from pymongo.errors import (
 from pymongo.operations import IndexModel, InsertOne
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import ReadPreference
+from pymongo.synchronous import client_session
+from pymongo.synchronous.client_session import TransactionOptions
+from pymongo.synchronous.command_cursor import CommandCursor
+from pymongo.synchronous.cursor import Cursor
+from pymongo.synchronous.helpers import next
 
-_TXN_TESTS_DEBUG = os.environ.get("TRANSACTION_TESTS_DEBUG")
+_IS_SYNC = True
 
 # Max number of operations to perform after a transaction to prove unpinning
 # occurs. Chosen so that there's a low false positive rate. With 2 mongoses,
@@ -60,19 +59,6 @@ UNPIN_TEST_MAX_ATTEMPTS = 50
 
 
 class TransactionsBase(SpecRunner):
-    @classmethod
-    def setUpClass(cls):
-        super().setUpClass()
-        if client_context.supports_transactions():
-            for address in client_context.mongoses:
-                cls.mongos_clients.append(single_client("{}:{}".format(*address)))
-
-    @classmethod
-    def tearDownClass(cls):
-        for client in cls.mongos_clients:
-            client.close()
-        super().tearDownClass()
-
     def maybe_skip_scenario(self, test):
         super().maybe_skip_scenario(test)
         if (
@@ -116,21 +102,20 @@ class TestTransactions(TransactionsBase):
     @client_context.require_transactions
     def test_transaction_write_concern_override(self):
         """Test txn overrides Client/Database/Collection write_concern."""
-        client = rs_client(w=0)
-        self.addCleanup(client.close)
+        client = self.rs_client(w=0)
         db = client.test
         coll = db.test
         coll.insert_one({})
         with client.start_session() as s:
             with s.start_transaction(write_concern=WriteConcern(w=1)):
-                self.assertTrue(coll.insert_one({}, session=s).acknowledged)
-                self.assertTrue(coll.insert_many([{}, {}], session=s).acknowledged)
-                self.assertTrue(coll.bulk_write([InsertOne({})], session=s).acknowledged)
-                self.assertTrue(coll.replace_one({}, {}, session=s).acknowledged)
-                self.assertTrue(coll.update_one({}, {"$set": {"a": 1}}, session=s).acknowledged)
-                self.assertTrue(coll.update_many({}, {"$set": {"a": 1}}, session=s).acknowledged)
-                self.assertTrue(coll.delete_one({}, session=s).acknowledged)
-                self.assertTrue(coll.delete_many({}, session=s).acknowledged)
+                self.assertTrue((coll.insert_one({}, session=s)).acknowledged)
+                self.assertTrue((coll.insert_many([{}, {}], session=s)).acknowledged)
+                self.assertTrue((coll.bulk_write([InsertOne({})], session=s)).acknowledged)
+                self.assertTrue((coll.replace_one({}, {}, session=s)).acknowledged)
+                self.assertTrue((coll.update_one({}, {"$set": {"a": 1}}, session=s)).acknowledged)
+                self.assertTrue((coll.update_many({}, {"$set": {"a": 1}}, session=s)).acknowledged)
+                self.assertTrue((coll.delete_one({}, session=s)).acknowledged)
+                self.assertTrue((coll.delete_many({}, session=s)).acknowledged)
                 coll.find_one_and_delete({}, session=s)
                 coll.find_one_and_replace({}, {}, session=s)
                 coll.find_one_and_update({}, {"$set": {"a": 1}}, session=s)
@@ -170,12 +155,11 @@ class TestTransactions(TransactionsBase):
     def test_unpin_for_next_transaction(self):
         # Increase localThresholdMS and wait until both nodes are discovered
         # to avoid false positives.
-        client = rs_client(client_context.mongos_seeds(), localThresholdMS=1000)
+        client = self.rs_client(client_context.mongos_seeds(), localThresholdMS=1000)
         wait_until(lambda: len(client.nodes) > 1, "discover both mongoses")
         coll = client.test.test
         # Create the collection.
         coll.insert_one({})
-        self.addCleanup(client.close)
         with client.start_session() as s:
             # Session is pinned to Mongos.
             with s.start_transaction():
@@ -198,12 +182,11 @@ class TestTransactions(TransactionsBase):
     def test_unpin_for_non_transaction_operation(self):
         # Increase localThresholdMS and wait until both nodes are discovered
         # to avoid false positives.
-        client = rs_client(client_context.mongos_seeds(), localThresholdMS=1000)
+        client = self.rs_client(client_context.mongos_seeds(), localThresholdMS=1000)
         wait_until(lambda: len(client.nodes) > 1, "discover both mongoses")
         coll = client.test.test
         # Create the collection.
         coll.insert_one({})
-        self.addCleanup(client.close)
         with client.start_session() as s:
             # Session is pinned to Mongos.
             with s.start_transaction():
@@ -259,7 +242,7 @@ class TestTransactions(TransactionsBase):
             return gfs.find(*args, **kwargs).next()
 
         def gridfs_open_upload_stream(*args, **kwargs):
-            bucket.open_upload_stream(*args, **kwargs).write(b"1")
+            (bucket.open_upload_stream(*args, **kwargs)).write(b"1")
 
         gridfs_ops = [
             (gfs.put, (b"123",)),
@@ -304,6 +287,14 @@ class TestTransactions(TransactionsBase):
                     "new-name",
                 ),
             ),
+            (
+                bucket.rename_by_name,
+                (
+                    "new-name",
+                    "new-name2",
+                ),
+            ),
+            (bucket.delete_by_name, ("new-name2",)),
         ]
 
         with client.start_session() as s, s.start_transaction():
@@ -327,11 +318,10 @@ class TestTransactions(TransactionsBase):
         # Start a transaction with a batch of operations that needs to be
         # split.
         listener = OvertCommandListener()
-        client = rs_client(event_listeners=[listener])
+        client = self.rs_client(event_listeners=[listener])
         coll = client[self.db.name].test
         coll.delete_many({})
         listener.reset()
-        self.addCleanup(client.close)
         self.addCleanup(coll.drop)
         large_str = "\0" * (1 * 1024 * 1024)
         ops: List[InsertOne[RawBSONDocument]] = [
@@ -356,13 +346,19 @@ class TestTransactions(TransactionsBase):
 
     @client_context.require_transactions
     def test_transaction_direct_connection(self):
-        client = single_client()
-        self.addCleanup(client.close)
+        client = self.single_client()
         coll = client.pymongo_test.test
 
         # Make sure the collection exists.
         coll.insert_one({})
         self.assertEqual(client.topology_description.topology_type_name, "Single")
+
+        def find(*args, **kwargs):
+            return coll.find(*args, **kwargs)
+
+        def find_raw_batches(*args, **kwargs):
+            return coll.find_raw_batches(*args, **kwargs)
+
         ops = [
             (coll.bulk_write, [[InsertOne[dict]({})]]),
             (coll.insert_one, [{}]),
@@ -379,16 +375,16 @@ class TestTransactions(TransactionsBase):
             (coll.count_documents, [{}]),
             (coll.distinct, ["foo"]),
             (coll.aggregate, [[]]),
-            (coll.find, [{}]),
+            (find, [{}]),
             (coll.aggregate_raw_batches, [[]]),
-            (coll.find_raw_batches, [{}]),
+            (find_raw_batches, [{}]),
             (coll.database.command, ["find", coll.name]),
         ]
         for f, args in ops:
             with client.start_session() as s, s.start_transaction():
                 res = f(*args, session=s)  # type:ignore[operator]
                 if isinstance(res, (CommandCursor, Cursor)):
-                    list(res)
+                    res.to_list()
 
 
 class PatchSessionTimeout:
@@ -407,6 +403,18 @@ class PatchSessionTimeout:
 
 
 class TestTransactionsConvenientAPI(TransactionsBase):
+    def setUp(self) -> None:
+        super().setUp()
+        self.mongos_clients = []
+        if client_context.supports_transactions():
+            for address in client_context.mongoses:
+                self.mongos_clients.append(self.single_client("{}:{}".format(*address)))
+
+    def set_fail_point(self, command_args):
+        clients = self.mongos_clients if self.mongos_clients else [self.client]
+        for client in clients:
+            self.configure_fail_point(client, command_args)
+
     @client_context.require_transactions
     def test_callback_raises_custom_error(self):
         class _MyException(Exception):
@@ -439,8 +447,7 @@ class TestTransactionsConvenientAPI(TransactionsBase):
     @client_context.require_transactions
     def test_callback_not_retried_after_timeout(self):
         listener = OvertCommandListener()
-        client = rs_client(event_listeners=[listener])
-        self.addCleanup(client.close)
+        client = self.rs_client(event_listeners=[listener])
         coll = client[self.db.name].test
 
         def callback(session):
@@ -468,8 +475,7 @@ class TestTransactionsConvenientAPI(TransactionsBase):
     @client_context.require_transactions
     def test_callback_not_retried_after_commit_timeout(self):
         listener = OvertCommandListener()
-        client = rs_client(event_listeners=[listener])
-        self.addCleanup(client.close)
+        client = self.rs_client(event_listeners=[listener])
         coll = client[self.db.name].test
 
         def callback(session):
@@ -501,8 +507,7 @@ class TestTransactionsConvenientAPI(TransactionsBase):
     @client_context.require_transactions
     def test_commit_not_retried_after_timeout(self):
         listener = OvertCommandListener()
-        client = rs_client(event_listeners=[listener])
-        self.addCleanup(client.close)
+        client = self.rs_client(event_listeners=[listener])
         coll = client[self.db.name].test
 
         def callback(session):
@@ -569,5 +574,29 @@ class TestTransactionsConvenientAPI(TransactionsBase):
             self.assertFalse(s.in_transaction)
 
 
+class TestOptionsInsideTransactionProse(TransactionsBase):
+    @client_context.require_transactions
+    @client_context.require_no_standalone
+    def test_case_1(self):
+        # Write concern not inherited from collection object inside transaction
+        # Create a MongoClient running against a configured sharded/replica set/load balanced cluster.
+        client = client_context.client
+        coll = client[self.db.name].test
+        coll.delete_many({})
+        # Start a new session on the client.
+        with client.start_session() as s:
+            # Start a transaction on the session.
+            s.start_transaction()
+            # Instantiate a collection object in the driver with a default write concern of { w: 0 }.
+            inner_coll = coll.with_options(write_concern=WriteConcern(w=0))
+            # Insert the document { n: 1 } on the instantiated collection.
+            result = inner_coll.insert_one({"n": 1}, session=s)
+            # Commit the transaction.
+            s.commit_transaction()
+        # End the session.
+        # Ensure the document was inserted and no error was thrown from the transaction.
+        assert result.inserted_id is not None
+
+
 if __name__ == "__main__":
     unittest.main()
